# Highlight search

搜索建议组件。

## 开始

安装组件：

```shell
 pnpm install @wankong/highlight-search 
```

## 参数说明

通过暴露的`createMEditor` 函数初始化。

`createMeditor`函数的参数说明：

```typescript
createMeditor({
	// 要挂载到的dom元素，可以直接设置dom元素，或者通过函数返回
	target:HTMLElement | ()=>HTMLElement
	suggestConfig:{
	   // 可以直接返回字段mapping数组或者通过Promise返回
	   mapping:FieldMapping[] | (() => Promise<FieldMapping[]>)
	   // 每个候选词的数组，需要根据该参数设置键盘的方向键移动位置
	   wordElement:()=>HTMLElement[]
	   // 查询 对应的搜索建议，仅在InputValue类型是会调用，且优先读缓存
	   // params 中有operator（运算符）,field（字段）和value（当前输入的值）
	   // options中有canalSignal，可用于终止axios或者fetch的请求，（内部自动处理了终止逻辑，仅绑定canalSignal即可）
	   fetchQuery:(params,options)=>Promise<FieldMapping[]>
	}
	onSubmit:(value:string)=>{
		// 编辑器触发搜索时的回调，value是纯文本字符串，等同于editor.getValue()
	}
})
```

`createMeditor`会返回editor实例，在原有TextBus的editor实例的基础上，还挂载了suggest对象和辅助函数。

suggest：搜索建议相关的所有东西都在这里处理。

Suggest对象上可能会用到的一些属性和函数：

| 名称             | 类型                       | 说明                                         |
| ---------------- | -------------------------- | -------------------------------------------- |
| loading          | boolean                    | 是否正在loading，目前仅在查询接口时会调用。  |
| showList         | FieldMapping[]             | 应该展示的候选词，已经过滤并且按原分组分割   |
| queryTypeByText  | (text:string)=>MappingType | 根据传入的单词判断该字符串的类型             |
| queryNextType    | (text:string)=>MappingType | 根据传入的单词推断下一个字符串的类型         |
| parseToFormatter | (text:string)=>Formatter[] | 根据传入的字符串返回解析格式之后的类型       |
| removeCache      | ()=>void                   | 清除缓存数据，建议每次提交检索之后都调用一次 |

editor实例上挂载的其他函数

| 名称     | 参数类型           | 说明                                                       |
| -------- | ------------------ | ---------------------------------------------------------- |
| choose   | (item:MappingItem) | 用于鼠标手动选中候选词的插入                               |
| setValue | (str:string)       | 手动设置编辑器的内容，会自动解析格式并格式化渲染到编辑器中 |
| getValue | -                  | 获取当前编辑器的内容，返回纯文本字符串                     |

## 自定义样式

插件并不会自带任何样式，但是预设了一套

## 使用示例

使用例子：

```vue
// 例如在 app.vue中
<template>
  <div class="editor-box">
    <HelloWorld msg="TextBUS Formatter here"></HelloWorld>
    <div class="tool-line">
      <button @click="handleSetValue()">设置默认值</button>
      <button @click="handleSearch()">获取输入的语句 </button>
      <span>{{ userInputValue }}</span>
      <span>loading:{{ meditor?.suggest.loading }}</span>
    </div>
    <div ref="editorRef" id="editor">
    </div>
    <!-- 以下内容可以任意写，搜索建议的数组是meditor?.suggest.showList,同时，loading和一些辅助函数也已经暴露出来了 -->
    <TransitionGroup tag="div" name="fade" class="suggest-list">
      <div v-if="meditor?.suggest.loading" :key="123">
        loading...
      </div>
      <div class="word-box" v-else-if="meditor?.suggest.showList.length"
        v-for="(item, index) in meditor?.suggest?.showList" :key="index">
        <div class="word-title">{{ item.title }} </div>
        <TransitionGroup tag="div" name="fade" class="words">
          <div class="word" v-for="(word, index) in item.mappings" :key="index" :class="{
            active: meditor?.suggest.activeItem?.value === word.value
          }" @click="handelChoose(word)">
            {{ item?.render?.(word) ?? word.name }}
          </div>
        </TransitionGroup>
      </div>
      <div :key="222" v-else>
        暂无搜索建议
      </div>
    </TransitionGroup>
  </div>
</template>

<script setup lang="ts" name="App">
import HelloWorld from "./components/HelloWorld.vue";
import { onMounted, ref } from "vue-demi";
import { createMEditor } from "./editor";
import { MEditor, MappingItem } from './editor/types/index.d'
import { parseFetchRes2FieldMapping } from "./editor/utils";
const editorRef = ref<HTMLElement>()
const meditor = ref<MEditor>()
const userInputValue = ref('')
const MODULE_NAME = "alarm"
const initEditor = async () => {
  const res = await fetch("/api/v1/sop/utils/xqlite/mappings/", {
    method: "POST",
    headers: {
      "Content-Type": "application/json",
    },
  })
  const info = await res.json()
  const mappings = parseFetchRes2FieldMapping(info.data, MODULE_NAME)
  meditor.value = await createMEditor(
    {
      target: editorRef.value!,
      suggestConfig: {
        mapping: mappings,
        wordElement: () => {
          return document.querySelectorAll('.word')
        },
        async fetchQuery(params, options) {
          const queryParams = {
            ...params,
            start_time: new Date("2022-12-01").getTime(),
            module: MODULE_NAME,
            end_time: new Date("2023-12-02").getTime()
          }
          try {
            const res = await fetch(`/api/v1/sop/utils/xqlite/suggest/`, {
              signal: options?.canalSignal,
              method: 'POST',
              headers: {
                'Content-Type': 'application/json'
              },
              body: JSON.stringify(queryParams)
            })
            const info = await res.json() as {
              data: string[]
            }
            return [{
              title: "搜索建议",
              mappings: info.data.map((item) => {
                return {
                  name: item,
                  value: `${item}`,
                  match: `${item}`
                }
              }),
              render(item) {
                return item.name
              },
            }]
          } catch (e) {
            console.warn("[meditor],fetchQuery error", e)
            return []
          }
        },
      },
      onSubmit: (value) => {
        console.log('[MEditor]:onSubmit,str is"', value + '"')
        alert(value)
      }
    })
}
onMounted(() => {
  initEditor()
})
const handelChoose = (word: MappingItem) => {
  meditor.value?.choose(word)
}
const handleSetValue = () => {
  const info = "source_ip = 123 and destination_ip_province = 123.2.2.2 or log_name exist"
  meditor.value?.setValue(info)
}
const handleSearch = () => {
  const str = meditor.value?.getValue()
  userInputValue.value = str ?? ''
}

</script>

<style scoped lang="scss">
.editor-box {
  display: flex;
  flex-direction: column;
  height: 100vh;
  width: 100vw;
  overflow: hidden;
}

#editor {
  flex-shrink: 0;
  text-align: left;
  width: 100%;
}

.tool-line {
  display: flex;
  align-items: center;
  justify-content: flex-start;
  margin-bottom: 20px;

  button {
    margin: 0 20px;
  }
}

.suggest-list {
  flex: 1 0 0;
  overflow-y: auto;
  margin-top: 10px;
  width: 100%;
  display: flex;
  flex-direction: column;
  text-align: left;
  position: relative;
  align-items: flex-start;

  .word-box {
    width: calc(100% - 40px);
    padding: 20px;
    background-color: #eeeeee;

    .word-title {
      text-align: left;
      font-size: 20px;
      font-weight: bold;
      flex-shrink: 0;
      margin-left: 10px;
    }

    .words {
      display: flex;
      flex-wrap: wrap;

      .word {
        margin: 10px;
        padding: 6px 10px;
        background-color: #fff;
        border-radius: 2px;
        cursor: pointer;

        &:hover {
          background-color: #cccccc;
          color: #ffffff;
        }

        &.active {
          background-color: #000000;
          color: #ffffff;
        }
      }
    }
  }
}

/* 1. 声明过渡效果 */
.fade-move,
.fade-enter-active,
.fade-leave-active {
  transition: all 0.5s cubic-bezier(0.55, 0, 0.1, 1);
}

/* 2. 声明进入和离开的状态 */
.fade-enter-from,
.fade-leave-to {
  opacity: 0;
  transform: scaleY(0.01) translate(30px, 0);
}

/* 3. 确保离开的项目被移除出了布局流
      以便正确地计算移动时的动画效果。 */
.fade-leave-active {
  position: absolute;
}
</style>

```





